/**
 * Copies the Location data (animation keyframes) from the first mask
 * of the selected video event's Bzier Masking FX to a selected
 * video event's Picture-In-Picture (PiP) FX
 *
 * To use this script:
 *
 * 1) Create track motion on an event using Bzier mask FX
 * 2) Apply a PiP FX to the event (from a different track) that you want to follow along with the motion on the original event
 * 3) Adjust location, scale and angle of PiP FX for a certain point in time.
 * 4) Select both events (hold ctrl key)
 * 5) Run the script
 *
 * Revision Date: Sep 5 2018
 * Copyright: MAGIX AG 
 **/
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Windows.Forms;
using ScriptPortal.Vegas;

public class EntryPoint
{
    Vegas myVegas;
    const string missingSelectionString = "Please select the video event to which you've applied tracking and the overlapping video event on a different track that holds the media you want to pin to the tracking.";
    const string missingTrackingDataString = "You must first apply the B\u00E9zier masking plug-in to one of the selected events and add motion tracking data for Mask 1.";
    const string missingPipFxString = "You must first add the Picture in Picture plug-in to the event that you want to pin to the motion tracked event.";

    public void FromVegas(Vegas vegas)
    {
        myVegas = vegas;
	string fxmask = "Location_0";

        using (MaskDialog MaskPrompt = new MaskDialog())
        {
             DialogResult result = MaskPrompt.ShowDialog();
             fxmask = MaskPrompt.getmask;
        }

        VideoEvent bezierTrack = null;
        OFXDouble2DParameter locationTracking = null;

        List<TrackEvent> trackEvents = FindAllSelectedEventsUnderCursor();

        if (trackEvents.Count < 2)
        {
            MessageBox.Show(missingSelectionString, "No selected event", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return;
        }
        else
        {
            bool hasBezierFx = false;
            foreach (VideoEvent trackEvent in trackEvents)
            {
                List<Effect> fxList = new List<Effect>(trackEvent.Effects);
                Effect bezierFX = fxList.Find(x => (x.PlugIn.UniqueID == "{Svfx:com.sonycreativesoftware:bzmasking}"));
                hasBezierFx = bezierFX != null;

                if (hasBezierFx)
                {
                    List<OFXParameter> ofxList = new List<OFXParameter>(bezierFX.OFXEffect.Parameters);
                    OFXParameter param = ofxList.Find(x => (x.Name == fxmask));

                    if (param != null)
                    {
                        locationTracking = (OFXDouble2DParameter)param;
                        bezierTrack = trackEvent;
                        trackEvents.Remove(trackEvent);
                        break;
                    }
                }
            }
            if (!hasBezierFx)
            {
                MessageBox.Show(missingTrackingDataString, "VEGAS B\u00E9zier Masking not applied", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
        }

        if (locationTracking == null || locationTracking.Keyframes.Count == 0)
        {
            MessageBox.Show(missingTrackingDataString, "No tracking data", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return;
        }
        else
        {
            bool hasPipFx = false;
            foreach (VideoEvent trackEvent in trackEvents)
            {
                List<Effect> fxList = new List<Effect>(trackEvent.Effects);
                Effect pipFX = fxList.Find(x => (x.PlugIn.UniqueID == "{Svfx:com.sonycreativesoftware:pictureinpicture}"));
                hasPipFx = pipFX != null;

                if (hasPipFx)
                {
                    Timecode startDiff = trackEvent.Start - bezierTrack.Start;

                    List<OFXParameter> ofxList = new List<OFXParameter>(pipFX.OFXEffect.Parameters);
                    OFXParameter param = ofxList.Find(x => (x.Name == "Location"));

                    if (param != null)
                    {
                        OFXDouble2DParameter pipLocation = (OFXDouble2DParameter)param;

                        OFXDouble2D initLocationAtCursorPosition = pipLocation.GetValueAtTime(myVegas.Transport.CursorPosition);
                        OFXDouble2D locationAtCursorPosition = locationTracking.GetValueAtTime(myVegas.Transport.CursorPosition);
                        OFXDouble2D diff = new OFXDouble2D();
                        diff.X = locationAtCursorPosition.X - initLocationAtCursorPosition.X;
                        diff.Y = locationAtCursorPosition.Y - initLocationAtCursorPosition.Y;

                        pipLocation.Keyframes.Clear();

                        foreach (var keyframe in locationTracking.Keyframes)
                        {
                            OFXDouble2D tmpValue = keyframe.Value;
                            tmpValue.X -= diff.X;
                            tmpValue.Y -= diff.Y;
                            pipLocation.SetValueAtTime(keyframe.Time - startDiff, tmpValue);
                        }
                    }
                }
                else
                {
                    MessageBox.Show(missingPipFxString, "No PiP FX", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
            }
        }
    }

    /// <summary>
    /// Returns the first selected event that's under the cursor
    /// </summary>
    /// <returns>The first selected event or null if no event selected</returns>
    List<TrackEvent> FindAllSelectedEventsUnderCursor()
    {
        List<TrackEvent> selectedEventsAcrossTracks = new List<TrackEvent>();

        foreach (Track track in myVegas.Project.Tracks)
        {
            foreach (TrackEvent trackEvent in track.Events)
            {
                if (trackEvent.Selected && trackEvent.Start <= myVegas.Transport.CursorPosition && trackEvent.End >= myVegas.Transport.CursorPosition)
                {
                    selectedEventsAcrossTracks.Add(trackEvent);
                }
            }
        }

        return selectedEventsAcrossTracks;
    }
}

   public class MaskDialog : Form
    {
 
        #region Vom Windows Form-Designer generierter Code

        /// <summary>
        /// Erforderliche Methode fr die Designeruntersttzung.
        /// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
        /// </summary>
        private void InitializeComponent()
        {
            this.button1 = new System.Windows.Forms.Button();
            this.button2 = new System.Windows.Forms.Button();
            this.button3 = new System.Windows.Forms.Button();
            this.button4 = new System.Windows.Forms.Button();
            this.button5 = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // button1
            // 
            this.button1.Location = new System.Drawing.Point(0, 0);
            this.button1.Name = "button1";
            this.button1.Size = new System.Drawing.Size(115, 23);
            this.button1.TabIndex = 0;
            this.button1.Text = "Mask 1";
            this.button1.UseVisualStyleBackColor = true;
            this.button1.Click += new System.EventHandler(this.button1_Click);
            // 
            // button2
            // 
            this.button2.Location = new System.Drawing.Point(0, 29);
            this.button2.Name = "button2";
            this.button2.Size = new System.Drawing.Size(115, 23);
            this.button2.TabIndex = 1;
            this.button2.Text = "Mask 2";
            this.button2.UseVisualStyleBackColor = true;
            this.button2.Click += new System.EventHandler(this.button2_Click);
            // 
            // button3
            // 
            this.button3.Location = new System.Drawing.Point(0, 58);
            this.button3.Name = "button3";
            this.button3.Size = new System.Drawing.Size(115, 23);
            this.button3.TabIndex = 2;
            this.button3.Text = "Mask 3";
            this.button3.UseVisualStyleBackColor = true;
            this.button3.Click += new System.EventHandler(this.button3_Click);
            // 
            // button4
            // 
            this.button4.Location = new System.Drawing.Point(0, 87);
            this.button4.Name = "button4";
            this.button4.Size = new System.Drawing.Size(115, 23);
            this.button4.TabIndex = 3;
            this.button4.Text = "Mask 4";
            this.button4.UseVisualStyleBackColor = true;
            this.button4.Click += new System.EventHandler(this.button4_Click);
            // 
            // button5
            // 
            this.button5.Location = new System.Drawing.Point(0, 116);
            this.button5.Name = "button5";
            this.button5.Size = new System.Drawing.Size(115, 23);
            this.button5.TabIndex = 4;
            this.button5.Text = "Mask 5";
            this.button5.UseVisualStyleBackColor = true;
            this.button5.Click += new System.EventHandler(this.button5_Click);
            // 
            // MaskDialog
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.ClientSize = new System.Drawing.Size(120, 146);
            this.Controls.Add(this.button5);
            this.Controls.Add(this.button4);
            this.Controls.Add(this.button3);
            this.Controls.Add(this.button2);
            this.Controls.Add(this.button1);
            this.Name = "MaskDialog";
            this.Text = "Form1";
            this.ResumeLayout(false);

        }

        #endregion

	// .................................
	private System.Windows.Forms.Button button1;
	private System.Windows.Forms.Button button2;
	private System.Windows.Forms.Button button3;
	private System.Windows.Forms.Button button4;
	private System.Windows.Forms.Button button5;

 	// .................................

        private string mask = "Location_0";

	public MaskDialog()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            mask = "Location_0";
	    Close ();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            mask = "Location_1";
		Close ();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            mask = "Location_2";
		Close ();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            mask = "Location_3";
		Close();
        }

        private void button5_Click(object sender, EventArgs e)
        {
            mask = "Location_4";
		Close();
        }

	public string getmask
	{
		get { return mask;}
        }
}
